library ieee;
use ieee.std_logic_1164.all;

library work;
use work.jddr_pkg.all;

entity jddr is
generic
(
	NUMPORTS : natural := 2;
	EXTRA_WRITE_LATENCY : natural := 0
);
port
(
	-- Connect to a global clock (0 or 1) that isn't already
	-- connected to another PLL
	tm4_glbclk	: in std_logic;
	tm4_devbus	: inout std_logic_vector(40 downto 0);
	
	-- PortN request signals @ individual user clocks
	clock		: in	std_logic_vector(NUMPORTS-1 downto 0);
	transrequest	: in	std_logic_vector(NUMPORTS-1 downto 0);
	transcomplete	: out	std_logic_vector(NUMPORTS-1 downto 0);
	ddr_address	: in	std_logic_vector(17*NUMPORTS-1 downto 0);
	ddr_start_block	: in	std_logic_vector(7*NUMPORTS-1 downto 0);
	ddr_end_block	: in	std_logic_vector(7*NUMPORTS-1 downto 0);
	readwriten	: in	std_logic_vector(NUMPORTS-1 downto 0);
	 
	-- PortN data signals @ 133mhz data_clk clock
	data_clk	: out	std_logic;
	read_data	: out	std_logic_vector(143 downto 0);
	write_data	: in	std_logic_vector(144*NUMPORTS-1 downto 0);
	data_available	: out	std_logic_vector(NUMPORTS-1 downto 0);
	data_wanted	: out	std_logic_vector(NUMPORTS-1 downto 0);
	
	-- DDR Module Interface Pin Signals
	tm4_ddr_DQ	: inout	std_logic_vector(71 downto 0);
	tm4_ddr_DQS	: inout	std_logic_vector(8 downto 0);
	tm4_ddr_DM	: out	std_logic_vector(8 downto 0);
	tm4_ddr_clk	: out	std_logic_vector(2 downto 0);
	tm4_ddr_A	: out	std_logic_vector(13 downto 0);
	tm4_ddr_RASn	: out	std_logic;
	tm4_ddr_CASn	: out	std_logic;
	tm4_ddr_WEn	: out	std_logic;
	tm4_ddr_Sn	: out	std_logic_vector(1 downto 0);
	tm4_ddr_BA	: out	std_logic_vector(1 downto 0);
	tm4_ddr_CLKE	: out	std_logic_vector(1 downto 0);
	tm4_ddr_RESETn	: out	std_logic;
	tm4_dqsclk_in	: in	std_logic;
	tm4_dqsclk_out	: out	std_logic
);
end jddr;

architecture rtl of jddr is
	signal r_transrequest	: std_logic;
	signal r_transcomplete	: std_logic;
	signal r_ddr_address	: std_logic_vector(16 downto 0);
	signal r_readwriten	: std_logic;
	signal r_ddr_start_block : std_logic_vector(6 downto 0);
	signal r_ddr_end_block	: std_logic_vector(6 downto 0);
	
	signal r_read_data	: std_logic_vector(143 downto 0);
	signal r_write_data	: std_logic_vector(143 downto 0);
	signal r_data_available	: std_logic;
	signal r_data_wanted	: std_logic;
	signal r_data_clk	: std_logic;
begin
	data_clk <= r_data_clk;
	
	interface : jddr_interface
	generic map
	(
		EXTRA_WRITE_DELAY => 3 + EXTRA_WRITE_LATENCY
	)
	port map
	(
		tm4_glbclk, tm4_devbus(40),
		r_transrequest, r_readwriten, r_ddr_address,
		r_transcomplete, r_ddr_start_block, r_ddr_end_block,
		r_read_data, r_write_data, r_data_available, r_data_wanted,
		r_data_clk,
		tm4_ddr_DQ, tm4_ddr_DQS, tm4_ddr_DM, tm4_ddr_clk, tm4_ddr_A,
		tm4_ddr_RASn, tm4_ddr_CASn, tm4_ddr_WEn, tm4_ddr_Sn,
		tm4_ddr_BA, tm4_ddr_CLKE, tm4_ddr_RESETn, tm4_dqsclk_in,
		tm4_dqsclk_out
	);

	parbiter : jddr_parbiter
	generic map
	(
		NUMPORTS => NUMPORTS
	)
	port map
	(
		clock, transrequest, transcomplete, ddr_address,
		ddr_start_block, ddr_end_block, readwriten,
		read_data, write_data, data_available, data_wanted,
		r_transrequest, r_transcomplete, r_ddr_address,
		r_ddr_start_block, r_ddr_end_block, r_readwriten,
		r_read_data, r_write_data, r_data_available, r_data_wanted,
		r_data_clk
	);
end rtl;
