library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;


entity jddr_datapath is
  port (
    -- globals
    clk            : in std_logic;
    reset_n        : in std_logic;
    dqs_outclk     : in std_logic;
    write_clk      : in std_logic;
    resynch_clk    : in std_logic;
    postamble_clk  : in std_logic;
    dqs_delay_ctrl : in std_logic;
    -- DDR Controller Signals
    doing_wr    : in  std_logic;
    wdata_low   : in  std_logic_vector(71 downto 0);
    wdata_high  : in  std_logic_vector(71 downto 0);
    doing_rd    : in  std_logic;
    rdata_low   : out std_logic_vector(71 downto 0);
    rdata_high  : out std_logic_vector(71 downto 0);
    -- DDR Interface Signals
    ddr_dq   : inout std_logic_vector(71 downto 0);
    ddr_dqs  : inout std_logic_vector(8 downto 0);
    ddr_dm   : out   std_logic_vector(8 downto 0)
  );
end jddr_datapath;

architecture rtl of jddr_datapath is
  component jddr_dqs_group port (
    -- globals
    clk            : in   std_logic;
    reset_n        : in   std_logic;
    dqs_outclk     : in std_logic;
    write_clk      : in   std_logic;
    resynch_clk    : in   std_logic;
    postamble_clk  : in   std_logic;
    dqs_delay_ctrl : in   std_logic;
    -- DDR Controller Signals
    doing_wr    : in   std_logic;
    wdata_low   : in   std_logic_vector(7 downto 0);
    wdata_high  : in   std_logic_vector(7 downto 0);
    doing_rd    : in   std_logic;
    rdata_low   : out  std_logic_vector(7 downto 0);
    rdata_high  : out  std_logic_vector(7 downto 0);
    -- DDR Interface Signals
    ddr_dq   : inout std_logic_vector(7 downto 0);
    ddr_dqs  : inout std_logic;
    ddr_dm   : out   std_logic
  );
  end component;

begin

  g_datapath : for i in 0 to 8 generate
    g_ddr_io : jddr_dqs_group
      port map (
         clk            => clk,
         reset_n        => reset_n,
         dqs_outclk     => dqs_outclk,
         write_clk      => write_clk,
         resynch_clk    => resynch_clk,
         postamble_clk  => postamble_clk,
         dqs_delay_ctrl => dqs_delay_ctrl,

         doing_wr    => doing_wr,
         wdata_low   => wdata_low((i+1)*8-1 downto i*8),
         wdata_high  => wdata_high((i+1)*8-1 downto i*8),
         doing_rd    => doing_rd,
         rdata_low   => rdata_low((i+1)*8-1 downto i*8),
         rdata_high  => rdata_high((i+1)*8-1 downto i*8),
         ddr_dq  => ddr_dq((i+1)*8-1 downto i*8),
         ddr_dqs => ddr_dqs(i),
         ddr_dm  => ddr_dm(i)
       );
    end generate;

end rtl;

